import logging
from threading import Thread

__all__ = ['logger', 'info', 'LOGDNA_API_KEY']
LOGDNA_API_KEY = '< YOUR INGESTION KEY HERE >'
logger = logging.getLogger('logdna')
logger.setLevel(logging.INFO)


def info(*args):
    def fn():
        for line in args:
            logger.info(line)

    thread = Thread(target=fn)
    thread.setDaemon(True)
    thread.start()
    return thread



import json
from os import path
from coverage import Coverage

ROOT = path.realpath(path.abspath(path.join(path.dirname(__file__), '..')))
COVERAGE_DIR = path.join(ROOT, 'coverage')
JUNIT_PATH = path.join(COVERAGE_DIR, 'test.xml')


def json_coverage():
    COVERAGE_FILE = path.join(COVERAGE_DIR, 'coverage-final.json')
    COVERAGE_SUMMARY = path.join(COVERAGE_DIR, 'coverage-summary.json')

    coverage = Coverage(config_file=path.join(ROOT, 'pyproject.toml'))
    coverage.load()
    coverage.json_report(outfile=COVERAGE_FILE)

    report = json.load(open(COVERAGE_FILE))
    totals = report.get('totals')
    summary = {
        'lines': {
            'total':
            totals['covered_lines'] + totals['missing_lines'],
            'covered':
            totals['covered_lines'],
            'pct':
            totals['covered_lines'] /
            (totals['covered_lines'] + totals['missing_lines']) * 100
        },
        'statements': {
            'total': None,
            'covered': None,
            'pct': None,
        },
        'functions': {
            'total': None,
            'covered': None,
            'pct': None,
        },
        'branches': {
            'total': totals['num_branches'],
            'covered': totals['covered_branches'],
            'pct': totals['covered_branches'] / totals['num_branches'] * 100
        }
    }

    json.dump({'total': summary}, open(COVERAGE_SUMMARY, 'w'))


if __name__ == "__main__":
    json_coverage()



import json
import logging
import requests
import socket
import sys
import threading
import time
from functools import reduce

from .configs import defaults
from .utils import sanitize_meta, get_ip


class LogDNAHandler(logging.Handler):
    def __init__(self, key, options={}):
        logging.Handler.__init__(self)
        self.internal_handler = logging.StreamHandler(sys.stdout)
        self.internal_handler.setLevel(logging.DEBUG)
        self.internalLogger = logging.getLogger('internal')
        self.internalLogger.setLevel(logging.DEBUG)
        self.internalLogger.addHandler(self.internal_handler)

        self.key = key
        self.buf = []
        self.secondary = []
        self.exception_flag = False
        self.flusher = None
        self.max_length = defaults['MAX_LINE_LENGTH']

        self.hostname = options.get('hostname', socket.gethostname())
        self.ip = options.get('ip', get_ip())
        self.mac = options.get('mac', None)
        self.level = options.get('level', 'info')
        self.verbose = str(options.get('verbose', 'true')).lower()
        self.app = options.get('app', '')
        self.env = options.get('env', '')
        self.url = options.get('url', defaults['LOGDNA_URL'])
        self.request_timeout = options.get('request_timeout',
                                           defaults['DEFAULT_REQUEST_TIMEOUT'])
        self.include_standard_meta = options.get('include_standard_meta',
                                                 False)
        self.index_meta = options.get('index_meta', False)
        self.flush_limit = options.get('flush_limit',
                                       defaults['FLUSH_BYTE_LIMIT'])
        self.flush_interval_secs = options.get('flush_interval',
                                               defaults['FLUSH_INTERVAL_SECS'])
        self.retry_interval_secs = options.get('retry_interval_secs',
                                               defaults['RETRY_INTERVAL_SECS'])
        self.tags = options.get('tags', [])
        self.buf_retention_byte_limit = options.get(
            'buf_retention_limit', defaults['BUF_RETENTION_BYTE_LIMIT'])
        self.user_agent = options.get('user_agent', defaults['USER_AGENT'])

        if isinstance(self.tags, str):
            self.tags = [tag.strip() for tag in self.tags.split(',')]
        elif not isinstance(self.tags, list):
            self.tags = []
        self.setLevel(logging.DEBUG)
        self.lock = threading.RLock()

    # TODO(esatterwhite): complexity too high (8)
    def buffer_log(self, message):
        if message and message['line']:
            if len(message['line']) > self.max_length:
                message['line'] = message[
                    'line'][:self.max_length] + ' (cut off, too long...)'
                if self.verbose in ['true', 'debug', 'd']:
                    self.internalLogger.debug(
                        'Line was longer than %s chars and was truncated.',
                        self.max_length)

        # Attempt to acquire lock to write to buf
        # otherwise write to secondary as flush occurs
        if self.lock.acquire(blocking=False):
            buf_size = reduce(lambda x, y: x + len(y['line']), self.buf, 0)

            if buf_size + len(message['line']) < self.buf_retention_byte_limit:
                self.buf.append(message)
            else:
                self.internalLogger.debug(
                    'The buffer size exceeded the limit: %s',
                    self.buf_retention_byte_limit)
            self.lock.release()

            if buf_size >= self.flush_limit and not self.exception_flag:
                self.flush()
        else:
            self.secondary.append(message)

        if not self.flusher:
            interval = (self.retry_interval_secs
                        if self.exception_flag else self.flush_interval_secs)
            self.flusher = threading.Timer(interval, self.flush)
            self.flusher.start()

    def clean_after_success(self):
        del self.buf[:]
        self.exception_flag = False
        if self.flusher:
            self.flusher.cancel()
            self.flusher = None

    def handle_exception(self, exception):
        if self.flusher:
            self.flusher.cancel()
            self.flusher = None
        self.exception_flag = True
        if self.verbose in ['true', 'error', 'err', 'e']:
            self.internalLogger.debug('Error sending logs %s', exception)

    # do not call without acquiring the lock
    def send_request(self):
        self.buf.extend(self.secondary)
        self.secondary = []
        data = {'e': 'ls', 'ls': self.buf}
        try:
            res = requests.post(url=self.url,
                                json=data,
                                auth=('user', self.key),
                                params={
                                    'hostname': self.hostname,
                                    'ip': self.ip,
                                    'mac': self.mac if self.mac else None,
                                    'tags': self.tags if self.tags else None
                                },
                                stream=True,
                                timeout=self.request_timeout,
                                headers={'user-agent': self.user_agent})
            res.raise_for_status()
            # when no RequestException happened
            self.clean_after_success()
        except requests.exceptions.RequestException as e:
            self.handle_exception(e)

    def flush(self):
        if len(self.buf) == 0:
            return
        if self.lock.acquire(blocking=False):
            self.send_request()
            self.lock.release()
        else:
            if not self.flusher:
                self.flusher = threading.Timer(1, self.flush)
                self.flusher.start()

    # TODO(esatterwhite): complexity too high (14)
    def emit(self, record):  # noqa: C901
        msg = self.format(record)
        record = record.__dict__
        opts = {}
        if 'args' in record:
            opts = record['args']
        if self.include_standard_meta:
            if isinstance(opts, tuple):
                opts = {}
            if 'meta' not in opts:
                opts['meta'] = {}
            for key in ['name', 'pathname', 'lineno']:
                opts['meta'][key] = record[key]

        message = {
            'hostname': self.hostname,
            'timestamp': int(time.time() * 1000),
            'line': msg,
            'level': record['levelname'] or self.level,
            'app': self.app or record['module'],
            'env': self.env
        }
        if not isinstance(opts, tuple):
            if 'level' in opts:
                message['level'] = opts['level']
            if 'app' in opts:
                message['app'] = opts['app']
            if 'hostname' in opts:
                message['hostname'] = opts['hostname']
            if 'env' in opts:
                message['env'] = opts['env']
            if 'timestamp' in opts:
                message['timestamp'] = opts['timestamp']
            if 'meta' in opts:
                if self.index_meta:
                    message['meta'] = sanitize_meta(opts['meta'])
                else:
                    message['meta'] = json.dumps(opts['meta'])
        self.buffer_log(message)

    def close(self):
        if len(self.buf) > 0:
            self.flush()
        logging.Handler.close(self)



import socket
from http.server import HTTPServer
from threading import Thread


def get_port():
    s = socket.socket(socket.AF_INET, type=socket.SOCK_STREAM)
    s.bind(('localhost', 0))
    _, port = s.getsockname()
    s.close()
    return port


def start_server(port, cls):
    server = HTTPServer(('localhost', port), cls)
    thread = Thread(target=server.handle_request)
    thread.setDaemon(True)
    thread.start()
    return thread



from .logdna import LogDNAHandler
__all__ = ['LogDNAHandler']

# Publish this class to the "logging.handlers" module so that it can be use
# from a logging config file via logging.config.fileConfig().
import logging.handlers

logging.handlers.LogDNAHandler = LogDNAHandler



import json
import unittest

from http.server import BaseHTTPRequestHandler
from logdna import LogDNAHandler
from .mock.server import get_port, start_server
from .mock.log import logger, info, LOGDNA_API_KEY

expectedLines = []


class SuccessfulRequestHandler(BaseHTTPRequestHandler):
    def do_POST(self):
        content_length = int(self.headers['Content-Length'])
        body = self.rfile.read(content_length)
        self.send_response(200)

        self.end_headers()
        body = json.loads(body)['ls']
        for keys in body:
            expectedLines.append(keys['line'])


class FailedRequestHandler(BaseHTTPRequestHandler):
    def do_POST(self):
        content_length = int(self.headers['Content-Length'])
        self.rfile.read(content_length)
        self.send_response(400)
        self.end_headers()


class LogDNAHandlerTest(unittest.TestCase):
    def server_recieves_messages(self):
        port = get_port()
        options = {
            'hostname': 'localhost',
            'url': 'http://localhost:{0}'.format(port),
            'ip': '10.0.1.1',
            'mac': 'C0:FF:EE:C0:FF:EE'
        }

        handler = LogDNAHandler(LOGDNA_API_KEY, options)
        logger.addHandler(handler)
        line = "python python python"

        server_thread = start_server(port, SuccessfulRequestHandler)
        logdna_thread = info(line)

        server_thread.join()
        logdna_thread.join()

        self.assertEqual(len(expectedLines), 1)
        self.assertIn(line, expectedLines)
        logger.removeHandler(handler)

    def messages_preserved_if_excp(self):
        port = get_port()
        options = {
            'hostname': 'localhost',
            'url': 'http://localhost:{0}'.format(port),
            'ip': '10.0.1.1',
            'mac': 'C0:FF:EE:C0:FF:EE'
        }

        handler = LogDNAHandler(LOGDNA_API_KEY, options)
        logger.addHandler(handler)
        line = "second test. server fails"

        server_thread = start_server(port, FailedRequestHandler)
        logdna_thread = info(line)

        server_thread.join()
        logdna_thread.join()

        self.assertEqual(len(handler.buf), 1)
        logger.removeHandler(handler)

    def stops_retention_when_buf_is_full(self):
        port = get_port()
        options = {
            'hostname': 'localhost',
            'url': 'http://localhost:{0}'.format(port),
            'ip': '10.0.1.1',
            'mac': 'C0:FF:EE:C0:FF:EE',
            'buf_retention_limit': 50,
            'equest_timeout': 10,
            'flush_interval': 1,
            'retry_interval_secs': 1
        }

        handler = LogDNAHandler(LOGDNA_API_KEY, options)
        logger.addHandler(handler)
        line = "when buffer grows bigger than we want"
        lineTwo = "when buffer grows bigger than we want. And more and more"

        server_thread = start_server(port, FailedRequestHandler)
        logdna_thread = info(line, lineTwo)

        server_thread.join()
        logdna_thread.join()

        self.assertEqual(len(handler.buf), 1)
        self.assertNotEqual(handler.buf[0]['line'], lineTwo)
        logger.removeHandler(handler)

    def test_run_tests(self):
        self.server_recieves_messages()
        self.messages_preserved_if_excp()
        self.stops_retention_when_buf_is_full()


if __name__ == '__main__':
    unittest.main()



import json
import socket


def is_jsonable(obj):
    try:
        json.dumps(obj)
        return True
    except (TypeError, OverflowError, ValueError):
        return False


def sanitize_meta(meta):
    keys_to_sanitize = []
    for key, value in meta.items():
        if not is_jsonable(value):
            keys_to_sanitize.append(key)
    if keys_to_sanitize:
        for key in keys_to_sanitize:
            del meta[key]
        meta['__errors'] = 'These keys have been sanitized: ' + ', '.join(
            keys_to_sanitize)
    return meta


def get_ip():
    s = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
    try:
        # doesn't even have to be reachable
        s.connect(('10.255.255.255', 1))
        ip = s.getsockname()[0]
    except Exception:
        ip = '127.0.0.1'
    finally:
        s.close()
    return ip



from setuptools import setup
from os import path, sep

# read the contents of your README file
this_directory = path.abspath(path.dirname(__file__))

with open(path.join(this_directory, 'README.md'), 'rb') as f:
    long_description = f.read().decode('utf-8')

kwargs = {"dir": this_directory, "sep": sep}
with open("{dir}{sep}logdna{sep}VERSION".format(**kwargs)) as f:
    version = f.read().strip('\n')

setup(
    name='logdna',
    packages=['logdna'],
    package_data={'': ['VERSION']},
    version=version,
    description='A Python Package for Sending Logs to LogDNA',
    author='LogDNA Inc.',
    author_email='help@logdna.com',
    license='MIT',
    url='https://github.com/logdna/python',
    download_url=('https://github.com/logdna/python/tarball/%s' % (version)),
    keywords=['logdna', 'logging', 'logs', 'python', 'logdna.com', 'logger'],
    install_requires=[
        'requests',
    ],
    classifiers=[
        'Topic :: System :: Logging',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
        'Programming Language :: Python :: 3.9'
    ],
    long_description=long_description,
    long_description_content_type='text/markdown',
)
